@@ -1,4 +1,4 @@
-require File.expand_path('../../../spec_helper', __FILE__)
+require_relative '../../spec_helper'
 
 ruby_version_is "2.4" do
   describe "Hash#transform_values" do
@@ -80,12 +80,12 @@ ruby_version_is "2.4" do
         @hash.freeze
       end
 
-      it "raises a RuntimeError on an empty hash" do
-        ->{ {}.freeze.transform_values!(&:succ) }.should raise_error(RuntimeError)
+      it "raises a #{frozen_error_class} on an empty hash" do
+        ->{ {}.freeze.transform_values!(&:succ) }.should raise_error(frozen_error_class)
       end
 
-      it "keeps pairs and raises a RuntimeError" do
-        ->{ @hash.transform_values!(&:succ) }.should raise_error(RuntimeError)
+      it "keeps pairs and raises a #{frozen_error_class}" do
+        ->{ @hash.transform_values!(&:succ) }.should raise_error(frozen_error_class)
         @hash.should == @initial_pairs
       end
 
